import os
import re
import colorama as color


class Tile:
    DIRECTORIES = ["earth nav data", "terrain", "textures"]

    lat = None
    long = None
    tile_dir = None
    tile_name = None
    water_only = False
    opts = {}

    def __init__(self, tile, dir, **opts):
        self.tile_name = tile
        self.tile_dir = os.path.join(dir, self.tile_name)
        (self.lat, self.long) = re.findall("([+-]\d+)", self.tile_name.lstrip("zOrtho4XP_").strip())
        self.verbose = opts["verbose"] if "verbose" in opts else 0
        self.errors = list()
        self.textures = dict()

    def validate(self):
        for dir in self.DIRECTORIES:
            getattr(self, "validate_{}".format(re.sub("\s+", "_", dir)))()

        return self.errors

    def validate_dir(self, dir):
        path = os.path.join(self.tile_dir, dir)

        if self.verbose > 2: print("  Validating {:.<35} ".format(dir.upper() + " directory"), end="")

        if not os.path.isdir(path):
            if dir != "terrain":
                if self.verbose > 2: print(color.Fore.RED + "NOT FOUND")
                self.errors.append("Directory \"{}\" for Tile {} NOT FOUND".format(dir, self.tile_name))

            return False

        if not os.listdir(path):
            if self.verbose > 2: print(color.Fore.RED + "IS EMPTY")
            self.errors.append("Directory \"{}\" for Tile {} IS EMPTY".format(dir, self.tile_name))
            return False

        if self.verbose > 2: print(color.Fore.GREEN + "OKAY")

        return True

    def validate_earth_nav_data(self):
        if not self.validate_dir("earth nav data"):
            return False

        return True

    def validate_terrain(self):
        no_errors_found = True
        path = os.path.join(self.tile_dir, "terrain")

        self.textures = dict(terrain_check=True)

        # May not exist (e.g. Water only tile)
        # if os.path.isdir(path):
        if not self.validate_dir("terrain"):
            self.water_only = True
            if self.verbose > 2: print(color.Fore.YELLOW + "Water Only? (if Textures are okay, this is safe to ignore)")
            return True

        if self.verbose > 2:
            print("  Validating {:.<35} ".format("TERRAIN DATA"), end=(os.linesep if self.verbose > 3 else ""))

        for terrain_file in sorted(os.listdir(path)):

            if self.verbose > 3: print("    Checking {:.<35} ".format(terrain_file), end="")

            texture_file = os.path.basename(
                re.search("../textures/(.+\.dds)", open(os.path.join(path, terrain_file)).read())[0])

            texture_path = os.path.join(self.tile_dir, "textures", texture_file)

            if not os.path.exists(texture_path):
                no_errors_found = False
                if self.verbose > 3: print(color.Fore.RED + "NO REFERENCE IN TEXTURES")
                if texture_file not in self.textures:
                    self.errors.append(
                        "Terrain for Tile {} points to {}, which does not exist".format(self.tile_name, texture_file))
            else:
                if self.verbose > 3: print(color.Fore.GREEN + "OKAY")

            self.textures[texture_file] = None

        if 2 < self.verbose <= 3:
            if no_errors_found:
                print(color.Fore.GREEN + "OKAY")
            else:
                print(color.Fore.RED + "ERROR")

        return no_errors_found

    def validate_textures(self):
        texture_count = 0
        no_errors_found = True
        path = os.path.join(self.tile_dir, "textures")

        if not self.validate_dir("textures"): return False

        if "terrain_check" not in self.textures: raise RuntimeError("Textures called before Terrains")

        if self.verbose > 2:
            print("  Validating {:.<35} ".format("TEXTURES DATA"), end=(os.linesep if self.verbose > 3 else ""))

        for texture_file in sorted(os.listdir(path)):
            if re.match(".*\.dds", texture_file) is None: continue

            texture_count += 1

            if self.verbose > 3: print("    Checking {:.<35} ".format(texture_file), end="")

            if texture_file not in self.textures:
                no_errors_found = False

                if self.verbose > 3: print(color.Fore.RED + "ERROR")

                if self.water_only:
                    if 2 < self.verbose <= 3:
                        print(color.Fore.RED + "ERROR")
                        
                    self.errors.append(
                        "Textures were found, but no TERRAIN directory exists for {}".format(self.tile_name))
                    return False  # no need to check the remaining files
                else:
                    self.errors.append(
                        "Texture file {} exists, but was not referenced by {}".format(texture_file, self.tile_name))
            else:
                if self.verbose > 3: print(color.Fore.GREEN + "OKAY")

        if texture_count == 0 and self.verbose > 3:
            if self.water_only:
                print("    {:.<44} {}".format("Water Only Tile", color.Fore.GREEN + "OKAY"))
            else:
                print("    {:.<44} {}".format("NO TEXTURE DATA", color.Fore.RED + "ERROR"))

        if 2 < self.verbose <= 3:
            if no_errors_found:
                print(color.Fore.GREEN + "OKAY")
            else:
                print(color.Fore.RED + "ERROR")

        return no_errors_found



import argparse
import os


class ArgParse:
    help = None
    args = None

    def __init__(self, **opts):
        parser = argparse.ArgumentParser(description="""Scan all Ortho4XP Tiles and report any problems""", **opts)

        # Positional Arguments
        parser.add_argument("tile_directory",
                            nargs="?",
                            default=".",
                            help="Directory where Tiles are stored (usually your Ortho4XP dir) - " +
                                 "If not provided; use the current directory")

        # Optional Arguments
        parser.add_argument("-P", "--pause", action="store_true",
                            help="Pause the program before exiting (good for batch files)")

        group = parser.add_mutually_exclusive_group()
        group.add_argument("-q", "--quiet", action="store_true",
                           help="Suppresses all output; exit value indicates errors found")
        group.add_argument("-v", "--verbose", dest="verbosity", action="count", default=1,
                           help="Increase output verbosity (may be repeated)")

        parser.add_argument("-V", "--version", action="version", version=opts["epilog"])

        self.args = parser.parse_args()
        self.help = parser.format_help()

        self.args.tile_directory = self.args.tile_directory.rstrip("/\\")

        if self.args.quiet: self.args.verbosity = 0



import atexit
import colorama as color
import os
import sys

from .options import ArgParse
from .globals import *
from .ortho import Tile
from .util import *


# Called during sys.exit()
def cleanup():
    color.deinit()  # Colorama


# Main

def main():
    errors = []
    bad_tiles = {}

    atexit.register(cleanup)

    color.init(autoreset=True)  # Colorama

    argparse = ArgParse(epilog=VERSION_STR, prog=PROGRAM)

    args = argparse.args
    help_message = argparse.help

    vquiet = args.verbosity == 0
    vnormal = args.verbosity == 1
    verbose = args.verbosity > 1

    if os.path.basename(args.tile_directory) == "Tiles":
        tiles_dir = args.tile_directory
    else:
        tiles_dir = os.path.join(args.tile_directory, "Tiles")

    if not os.path.isdir(tiles_dir):
        usage(help_message, "Please provide a directory where Ortho4XP Tiles can be found")

    tiles = sorted(os.listdir(tiles_dir))
    tiles_count = len(tiles)

    if tiles_count == 0:
        usage(help_message,
              "\"{}\" does not appear to be a valid Ortho4XP Tiles directory, or if it is, I cannot find any Tiles."
              .format(tiles_dir))

    if not vquiet: print(VERSION_STR + os.linesep)

    if vnormal:
        # ToDo: Create a progress bar for this
        print(color.Fore.LIGHTCYAN_EX + "Analyzing Tiles... this may take some time, please wait...")
        print("(Optionally, you can CTRL-C now and use the --verbose option to see the progress)")

    # Run the validations for each Tile
    for tile in tiles:
        err_count = len(errors)

        if verbose:
            if args.verbosity > 2: print()
            print("Analyzing Tile {:.<33} ".format(tile), end=(os.linesep if args.verbosity > 2 else ""))

        errors.extend(Tile(tile, dir=tiles_dir, verbose=args.verbosity).validate())

        if len(errors) == err_count:
            if args.verbosity == 2: print(color.Fore.GREEN + "OKAY")
        else:
            if args.verbosity == 2: print(color.Fore.RED + "ERROR")
            bad_tiles[tile] = (len(errors) - err_count)

    err_count = len(errors)

    if args.verbosity > 0:
        print(os.linesep + "Scanned {}... ".format(pluralize(tiles_count, "Tile")), end="")

        if err_count == 0:
            print(color.Fore.LIGHTGREEN_EX + "All OKAY")
        else:
            bad_tiles_count = len(bad_tiles)
            bad_tiles_string = pluralize(bad_tiles_count, "tile")
            err_count_string = pluralize(err_count, "error")

            print(color.Fore.LIGHTRED_EX + "Found {} with {} in total".format(
                bad_tiles_string, err_count_string
            ))

            with open("{}.error.log".format(PROGRAM_SHORT), "w") as f:
                print(os.linesep + "{} written to {}".format(err_count_string, f.name))
                for error in errors: f.write(error + os.linesep)

            print(os.linesep + "{} need attention".format(bad_tiles_string), file=sys.stderr)
            for (bad_tile_name, bad_tile_count) in bad_tiles.items():
                print("  -> {} ({})".format(
                    color.Fore.LIGHTWHITE_EX + bad_tile_name, pluralize(bad_tile_count, "error")),
                    file=sys.stderr
                )

    if args.pause:
        input(os.linesep + "Press ENTER key to exit")

    sys.exit(err_count)


if __name__ == "__main__": main()



import colorama as color
import sys

__all__ = ["pluralize", "usage"]


def pluralize(count, singular, plural=None):
    if plural is None:
        plural = singular + "s"

    if count == 0:
        return "{} {}".format("no", plural)

    if count == 1:
        return "{} {}".format(count, singular)

    return "{} {}".format(count, plural)


def usage(help_message, error_message=""):
    if error_message != "": print(color.Fore.RED + color.Style.BRIGHT + f"!!! {error_message} !!!\n", file=sys.stderr)
    print(help_message)

    sys.exit(255)



from codecs import open
from os import path
from otv.globals import VERSION
from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="otv",
    version=VERSION,
    license='MIT',

    description="Ortho4XP Tile Validator",
    long_description=long_description,
    url="https://github.com/dyoung522/otv",

    author="Donovan C. Young",
    author_email="dyoung522@gmail.com",

    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Games/Entertainment",
        "Topic :: Games/Entertainment :: Simulation",
        "Topic :: Utilities",
    ],
    keywords=["ortho4xp", "x-plane"],

    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=['colorama'],
    entry_points={'console_scripts': ['otv=otv:main']},
)

